// gpio.c

#include "gpio.h"  

VideoParam encoder_params[] =
{ 
//  Name                           addr   val (use 0x100 to skip)
  { "Mode Register 0"            , 0x00,  0x05, 0 }, // Extended Luminance 1.3MHz PAL mode
  { "Mode Register 1"            , 0x01,  0x3F, 0 }, // DACs operational 2x oversampling
  { "Mode Register 2"            , 0x02,  0x48, 0 }, // Normal operation R,G,B, CVBS, Luma,
                                                     // Chroma output with pedestal NTSC/PAL input IGNORED
  { "Mode Register 3"            , 0x03,  0x00, 0 }, // No Teletext or Closed Caption Data
  { "Mode Register 4"            , 0x04,  0x00, 0 }, // Interlaced  Output 720 Active Pixels
  { "Mode Register 5"            , 0x05,  0x09, 0 }, // Sync on RGB outputs, SMPTE output levels
  { "Mode Register 6"            , 0x06,  0x01, 0 }, // Power up sleep mode disabled
  { "Mode Register 7"            , 0x07,  0x00, 0 }, // Color Controls are Disabled
  { "Mode Register 8"            , 0x08,  0x10, 0 }, // 10 Bit Pixel data Interlaced output NO GAMMA controls
  { "Mode Register 9"            , 0x09,  0x00, 0 }, // No Chroma Delay 
  { "Timing Register 0"          , 0x0A,  0x00, 0 }, // CCIR 656 Slave mode all timing SAV, EAV time codes, BLANK enabled
  { "Timing Register 1"          , 0x0B,  0x00, 0 }, // Normal Operation
  { "Subcarrier Freq Reg 0"      , 0x0C,  0xCB, 0 }, // Subcarrier for PAL
  { "Subcarrier Freq Reg 1"      , 0x0D,  0x8A, 0 }, // Subcarrier for PAL
  { "Subcarrier Freq Reg 2"      , 0x0E,  0x09, 0 }, // Subcarrier for PAL
  { "Subcarrier Freq Reg 3"      , 0x0F,  0x2A, 0 }, // Subcarrier for PAL
  { "Subcarrier Phase Reg"       , 0x10,  0x00, 0 }, // No added color phase shift
  { "Closed Caption Ext Reg 0"   , 0x11,  0x00, 0 }, // No CC data
  { "Closed Caption Ext Reg 1"   , 0x12,  0x00, 0 }, // No CC data
  { "Closed Caption  Reg 0"      , 0x13,  0x00, 0 }, // No CC data
  { "Closed Caption  Reg 1"      , 0x14,  0x00, 0 }, // No CC data
  { "Pedestal Control Reg 0"     , 0x15,  0x00, 0 }, // Pedestal Turned on  for CC lines 
  { "Pedestal Control Reg 1"     , 0x16,  0x00, 0 }, // Pedestal Turned on  for CC lines 
  { "Pedestal Control Reg 2"     , 0x17,  0x00, 0 }, // Pedestal Turned on  for CC lines 
  { "Pedestal Control Reg 3"     , 0x18,  0x00, 0 }, // Pedestal Turned on  for CC lines 
  { "CGMS_WSS Reg 0"             , 0x19,  0x00, 0 }, // No CGMS data
  { "CGMS_WSS Reg 1"             , 0x1A,  0x00, 0 }, // No CGMS data
  { "CGMS_WSS Reg 2"             , 0x1B,  0x00, 0 }, // No CGMS data
  { "Teletext Control"           , 0x1C,  0x00, 0 }, // No Teletext data 
  { "Contrast Control"           , 0x1D,  0x00, 0 }, // No modification of Y
  { "Color Control Reg 0"        , 0x1E,  0x00, 0 }, // No modification of  U
  { "Color Control Reg 1"        , 0x1F,  0x00, 0 }, // No modification of  V
  { "Hue Control Reg"            , 0x20,  0x00, 0 }, // No modification to C or Comp Video
  { "Brightness Control"         , 0x21,  0x00, 0 }, // No modification  to Y or Comp Video Brightness
  { "Sharpness Response"         , 0x22,  0x06, 0 }, // No boost or cut of Luma filter response 
  { "DNR 0"                      , 0x23,  0x44, 0 }, // Add a little gain for DNR
  { "DNR 1"                      , 0x24,  0x20, 0 }, // 8 Pixel Block with 2 Pixel Border for DNR
  { "DNR 2"                      , 0x25,  0x00, 0 }, // Normal DNR operation
  { "Gamma 0"                    , 0x26,  0x13, 0 }, // Curve A Y32 correction
  { "Gamma 1"                    , 0x27,  0x20, 0 }, // Curve A Y64 correction
  { "Gamma 2"                    , 0x28,  0x37, 0 }, // Curve A Y96 correction
  { "Gamma 3"                    , 0x29,  0x55, 0 }, // Curve A Y128 correction
  { "Gamma 4"                    , 0x2A,  0x7A, 0 }, // Curve A Y160 correction
  { "Gamma 5"                    , 0x2B,  0x75, 0 }, // Curve A Y192 correction
  { "Gamma 6"                    , 0x2C,  0xD5, 0 }, // Curve A Y224 correction
  { "Gamma 7"                    , 0x2D,  0x11, 0 }, // Curve B Y32 correction
  { "Gamma 8"                    , 0x2E,  0x18, 0 }, // Curve B Y64 correction
  { "Gamma 9"                    , 0x2F,  0x27, 0 }, // Curve B Y96 correction
  { "Gamma 10"                   , 0x30,  0x41, 0 }, // Curve B Y128 correction
  { "Gamma 11"                   , 0x31,  0x65, 0 }, // Curve B Y160 correction
  { "Gamma 12"                   , 0x32,  0x94, 0 }, // Curve B Y192 correction
  { "Gamma 13"                   , 0x33,  0xCE, 0 }, // Curve B Y224 correction
  { "Brightness Detect"          , 0x34, 0x100, 0 }, // READ ONLY
  { "Output Clock"               , 0x35,  0x72, 0 }, // Output Clock is DISABLED 
  { 0                            , 0x00,  0x00, 0 }  // End of list marker
};

VideoParam decoder_params[] =
{ 
  { "Input Control"                  , 0x00,  0x80, 0 }, // Force Composite PAL on input 1
  { "Video Selection"                , 0x01,  0x03, 0 }, // VCR Quality Video
  { "Video Enhancement"              , 0x02,  0x14, 0 }, // Flat Filter Response Truncate black if Y < +16
  { "Output Control"                 , 0x03,  0x00, 0 }, // Line Locked Clock Interface 10 bit 4:2:2 @ 27MHz
  { "Extended Output Control"        , 0x04,  0x8C, 0 }, // Luminance Data Range 16-235Chrominance Data Range 16-240
  // Take pwr on default for rest
#if 0
  { "General Purpose Output"         , 0x05,  0x40, 0 }, // Blank Cr & Cb during VBI, GPOs are tristated
  { "RESERVED"                       , 0x06, 0x100, 0 }, // READ ONLY AND IGNORE
  { "FIFO Control"                   , 0x07,  0x90, 0 }, // AFF & AEF become active 16 locations before full and empty
  { "Contrast Control"               , 0x08,  0x80, 0 }, // Luminance Gain = 1
  { "Saturation Control"             , 0x09,  0x80, 0 }, // Color Gain = 1
  { "Brightness Control"             , 0x0A,  0x00, 0 }, // No Additional Brightness Added
  { "Hue Control"                    , 0x0B,  0x00, 0 }, // No Hue Change
  { "Default Y Value"                , 0x0C,  0x10, 0 }, // Use programmed Y,Cr,Cb values if lock is lost (default Y=16)
  { "Default C Value"                , 0x0D,  0x88, 0 }, // Default Cr=128, Default Cb=128 (Defaults to a Black Screen)
  { "Temporal Decimation"            , 0x0E,  0x00, 0 }, // Suppress no frames, start with even field
  { "Power Management"               , 0x0F,  0x00, 0 }, // Normal Operation
  { "Status Register"                , 0x10, 0x100, 0 }, // READ ONLY AND IGNORE
  { "Info Register"                  , 0x11, 0x100, 0 }, // READ ONLY AND IGNORE
  { "RESERVED"                       , 0x12, 0x100, 0 }, // READ ONLY AND IGNORE
  { "Analog Control Internal"        , 0x13,  0x45, 0 }, // H-Synch V-Synch and field are output based on OE
  { "Analog Clamp Control"           , 0x14,  0x18, 0 }, // Voltage Clamp DISABLED Current Clamp ENABLED
  { "Digital Clamp Control 1"        , 0x15,  0x60, 0 }, // Automatic Digital Clamp based on Video Quality (reg 01)
  { "Digital Clamp Control 2"        , 0x16,  0x00, 0 }, // No Offset added
  { "Shaping Filter Control"         , 0x17,  0x17, 0 }, // PAL Filter Selection WIDE
  { "RESERVED"                       , 0x18, 0x100, 0 }, // READ ONLY AND IGNORE
  { "Misc Control"                   , 0x19,  0x08, 0 }, // 2 line non-adaptive Chroma comb filter
  { "Scaling/Cropping"               , 0x1A,  0x90, 0 }, // Scaling and Cropping Disabled for PAL
  { "Active Video Desired Lines"     , 0x1B,  0x1F, 0 }, // 240 Lines for PAL
  { "Active Video Vertical Begin"    , 0x1C,  0x17, 0 }, // Line 22 for PAL
  { "Vertical Scale Value 1"         , 0x1D,  0x01, 0 }, // Vertical Scaling Factor of 1
  { "Vertical Scale Value 2"         , 0x1E,  0x00, 0 }, // Vertical Scaling Factor of 1
  { "Active Video HorizontalBegin"   , 0x1F,  0x42, 0 }, // Pixel 61 for PAL
  { "Active Video Desired Pixel"     , 0x20,  0xD0, 0 }, // 720 Active Pixels per Line
  { "Horizontal Scale Value 1"       , 0x21,  0x09, 0 }, // Horizontal Scaling Factor of 1
  { "Horizontal Scale Value 2"       , 0x22,  0x7B, 0 }, // Horizontal Scaling Factor of 1 for PAL
  { "Color Subcarrier Control 1"     , 0x23,  0xE0, 0 }, // Manual Subcarrier DISABLED
  { "Color Subcarrier Control 2"     , 0x24,  0xA0, 0 }, // Subcarrier for PAL
#endif
  { 0                                , 0x00,  0x00, 0 }  // End of list marker
};
unsigned long gpio_shadow_register = 0;

void enable_i2c()
{ gpio_shadow_register |= (I2C_ENABLE_BIT);
  *GPIOADDR = gpio_shadow_register;
}

void disable_i2c()
{ gpio_shadow_register &= ~(I2C_ENABLE_BIT);
  *GPIOADDR = gpio_shadow_register;
}

void set_i2c_clock()
{ gpio_shadow_register |= (I2C_CLOCK_BIT);
  *GPIOADDR = gpio_shadow_register;
}

void clear_i2c_clock()   
{ gpio_shadow_register &= ~(I2C_CLOCK_BIT);
  *GPIOADDR = gpio_shadow_register;
}

void set_i2c_data()
{ gpio_shadow_register |= (I2C_DATA_BIT);
  *GPIOADDR = gpio_shadow_register;
}

void clear_i2c_data()
{ gpio_shadow_register &= ~(I2C_DATA_BIT);
  *GPIOADDR = gpio_shadow_register;
}

int test_i2c_clock()
{ return  ((*GPIOADDR) & I2C_CLOCK_BIT) != 0 ? 1 : 0;
}

int test_i2c_data()
{ return ((*GPIOADDR) & I2C_DATA_BIT ) != 0 ? 1 : 0;
}

static void i2c_delay()
{ int i = 0;
  while( ++i < 600)
  {
    // loop
  }
}

static int i2c_wait_on_clock()
{ int i = 0;

  while( ++i < 4000000 )
  {
    if( test_i2c_clock() )
      return 1;
  }

  return 0;
}

#define SCL_H()   clear_i2c_clock()
#define SCL_L()   set_i2c_clock()
#define SDA_H()   clear_i2c_data()
#define SDA_L()   set_i2c_data()
#define SCL_IN()  test_i2c_clock()
#define SDA_IN()  test_i2c_data()

/*
 * Makes sure that the bus is in a known condition. Returns 1 on success,
 * 0 if some other device is pulling on the bus.
 */

static int i2c_reset(void)
{
  SCL_H();
  SDA_H();
  enable_i2c();
  return (SCL_IN() && SDA_IN());
}


/*
 * Generates a start condition on the bus. Returns 0 on success, 1 if some
 * other device is holding the bus.
 */

static int i2c_start(void)
{ // DBP added - make sure clock is high
  i2c_delay();
  SCL_H();

  i2c_delay();
  SDA_L();        /* Pull SDA down... */
  i2c_delay();
  SCL_L();        /* ...and then SCL -> start condition. */
  i2c_delay();
  return 0;
}

static void SDA_OUT( int bit )
{ if( bit ) SDA_H();
  else SDA_L();
}

/*
 * Generates a stop condition on the bus. Returns 1 on success, 0 if some
 * other device is holding the bus.
 */

static int i2c_stop(void)
{
  SDA_L();
  i2c_delay();

  SCL_H();        /* Let SCL go up */
  i2c_delay();
  SDA_H();        /* ...and then SDA up -> stop condition. */
  i2c_delay();
    
  return (SCL_IN() && SDA_IN());  /* Both will be up, if everything is fine */
}

/*
 * Clock out one bit.
 * Returns 0 on success, 1 if we lose arbitration.
 */

static int i2c_bit_out(int bout)
{
  SDA_OUT(bout);              /* Put data out on SDA */
  i2c_delay();
  SCL_H();                    /* Let SCL go up */

  if(!i2c_wait_on_clock() )   /* Wait until all other devices are ready */
    return 1;
        
  if( SDA_IN() != bout )      /* Arbitration lost, release bus and return */
  {
    SDA_H();                  /* Should be up anyway, but make sure */
    return 1;
  }

  i2c_delay();
  SCL_L();                    /* Pull SCL back down */
  i2c_delay();                
  return 0;                   /* OK */
}


/*
 * Clock in one bit.
 */

static int i2c_bit_in(void)
{
  int bin;

  SCL_H();                    /* Let SCL go up */
  if( !i2c_wait_on_clock() )  /* Wait for other devices */
    return 1;

  bin = SDA_IN();             /* Read in data */
  i2c_delay();
  SCL_L();                    /* Pull SCL back down */
  i2c_delay();
  return bin;                 /* Return the sampled bit */
}


/*
 * Send one byte on the bus. No start or stop conditions are generated here,
 * but i2c_error will be set according to the result.
 * Returns 0 on success, 1 if we lose arbitration or if the slave doesn't
 * acknowledge the byte. Check i2c_error for the actual result on error.
 */

static int i2c_byte_out(int dat)
{
  int bit_count = 8;

  // Clock is low here

  while(bit_count)
  {
    if (dat & 0x80)
    {
      if ( i2c_bit_out(1) ) return 1;
    }
    else
    {
      if (i2c_bit_out(0)) return 1;
    }

    dat <<= 1;
    bit_count--;
  }

  // Added by DBP - free the data bus so that slave can drive ACK
  SDA_H();
  i2c_delay();

  // Get ACK 
  if( i2c_bit_in() )
  {
    return 1;
  }

  return 0;
}


/*
 * Reads one byte in from the slave. Ack must be 1 if this is the last byte
 * to be read during this transfer, 0 otherwise (as per I2C bus specification,
 * the receiving master must acknowledge all but the last byte during a
 * transfer).
 */
 
static int i2c_byte_in( int ack )
{
  int bit_count, byte_in;

  // Added by DBP - free the data bus so that slave can drive ACK
  SDA_H();
  i2c_delay();

  bit_count = 8;
  byte_in = 0;
    
  while( bit_count )
  {
    byte_in <<= 1;
    if( i2c_bit_in() ) byte_in |= 0x01;
      bit_count--;
  }

  i2c_bit_out( ack );
  SDA_H();

  return byte_in;
}

static int read_i2c_reg( int decoder, int addr )
{ int reva_detect = 0;
  int val;
  int reva, revc;

  if( decoder )
  { reva = decoder_addr_reva;
    revc = decoder_addr_revc;
  }
  else
  { reva = encoder_addr_reva;
    revc = encoder_addr_revc;
  }

  // Make sure everything is high
  if( !i2c_reset() )
  { // Can't free bus
    return -1;
  }

  // Data goes low, then clock
  i2c_start();

  // Now both clock, data are low

  // Try Rev C, then Rev A master address
  if( i2c_byte_out( revc ) )
  { i2c_stop();
    i2c_delay();
    i2c_delay();
    i2c_start();
    if( i2c_byte_out( reva ) )
    { i2c_stop();
      return -1;
    }
    reva_detect = 1;
  }

  // Put out sub-address
  if( i2c_byte_out( addr ) )
  { i2c_stop();
    return -1;
  }

  i2c_start();

  if( i2c_byte_out( (reva_detect ? reva : revc ) | 1 ) )
  { i2c_stop();
    return -1;
  }

  val = i2c_byte_in( 1 );
  i2c_stop();

  return val;
}

int write_i2c_reg( int decoder, int addr, int val )
{ int reva, revc;

  if( decoder )
  { reva = decoder_addr_reva;
    revc = decoder_addr_revc;
  }
  else
  { reva = encoder_addr_reva;
    revc = encoder_addr_revc;
  }

  // Make sure everything is high
  if( !i2c_reset() )
  { // Can't free bus
    return -1;
  }

  // Data goes low, then clock
  i2c_start();

  // Now both clock, data are low

  // Try Rev C, then Rev A master address
  if( i2c_byte_out( revc ) )
  { i2c_stop();
    i2c_delay();
    i2c_delay();
    i2c_start();
    if( i2c_byte_out( reva ) )
    { i2c_stop();
      return -1;
    }
  }

  // Put out sub-address
  if( i2c_byte_out( addr ) )
  { i2c_stop();
    return -1;
  }

  // Put out value
  if( i2c_byte_out( val ) )
  { i2c_stop();
    return -1;
  }

  i2c_stop();

  return 0;
}

int program_video( void )
{ int hw;
  int i;
  int err = 0;

  VideoParam *p;

  for( hw = 0; hw<2; hw ++ )
  {
    p = hw ? decoder_params : encoder_params;

    for( i=0; p[i].name; i++ )
    {
      if( p[i].def > 0xff ) continue;

      write_i2c_reg( hw, p[i].addr, p[i].def );
      p[i].value = read_i2c_reg( hw, p[i].addr );

      if( p[i].def != p[i].value )
      {
        err = 1;
      }
    }
  }

  return err;
}
