----------------------------------------------------------------------
--
-- Filename: gpio2iic.vhd
--
-- Description:
--
--
--                  Author: Mark Paluszkiewicz
--                  Staff  Engineer
--
--                  Xilinx Labs
--                  Xilinx, Inc.
--
--                  Copyright (c) 2001-2002 Xilinx, Inc.
--                  All rights reserved
--
--                  Date:   July 1 2002
--                  For:    JPEG2000 Encoder
--
--                  RESTRICTED RIGHTS LEGEND
--
--      This software/hdl has not been published by the author, and
--      has been disclosed to others for the purpose of enhancing
--      and promoting design productivity in Xilinx products.
--
--      Therefore use, duplication or disclosure, now and in the
--      future should give consideration to the productivity
--      enhancements afforded the user of this code by the author's
--      efforts.  Thank you for using our products !
--
-- Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--              WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY
--              IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--              A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--
--
-- Revision:
--      07/01/2002 MP Created this version Un-released
--
--
----------------------------------------------------------------------

library IEEE;
--library virtex2;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_UNSIGNED.all;
--use virtex2.components.all;
-- synthesis translate_off
library unisim;
use unisim.vcomponents.all;
-- synthesis translate_on
entity gpio2iic is
port (
  gpio_data_i2                 : out std_logic;
  gpio_data_i3                 : out std_logic;
  gpio_data_o2                 : in  std_logic;
  gpio_data_o3                 : in  std_logic;
  gpio_data_o4                 : in  std_logic;
  video_encoder_sclk_p           : inout std_logic;
  video_encoder_data_p           : inout std_logic;
  i2c_clock_p                    : inout std_logic;
  i2c_data_p                     : inout std_logic
);
end gpio2iic;

 architecture STRUCT of gpio2iic is

component IOBUF_S_12
      port (I, T: in std_logic;
            O: out std_logic;
            IO: inout std_logic);
end component;



 signal  gnd                      : std_logic;
 signal  enable_iic_outputs       : std_logic;
 signal  output_enables_iic_data  : std_logic;
 signal  output_enables_iic_clock : std_logic;
 signal  iic_clocks               : std_logic;
 signal  iic_datas                : std_logic;
 signal  video_encoder_sclk_i   : std_logic;
 signal  i2c_clock_i            : std_logic;
 signal  video_encoder_data_i   : std_logic;
 signal  i2c_data_i             : std_logic;
 signal  video_encoder_sclk_t   : std_logic;
 signal  i2c_clock_t            : std_logic;
 signal  video_encoder_data_t   : std_logic;
 signal  i2c_data_t             : std_logic;

------------------------------------------------------------------------------------------------------------------------------------------------

begin
 gnd <= '0';
 iic_clocks               <= video_encoder_sclk_i and i2c_clock_i;
 iic_datas                <= video_encoder_data_i and i2c_data_i ;
 enable_iic_outputs       <= gpio_data_o4;
 output_enables_iic_clock <= gpio_data_o3;                       -- to make it easy to figure which bus bits connect to the clock
 output_enables_iic_data  <= gpio_data_o2;                       -- to make it easy to figure which bus bits connect to the data
 gpio_data_i3           <= iic_clocks;                           -- to make it easy to figure which bus bits connect to the clock
 gpio_data_i2           <= iic_datas;                            -- to make it easy to figure which bus bits connect to the data
 video_encoder_sclk_t     <= not (enable_iic_outputs  and output_enables_iic_clock );
 video_encoder_data_t     <= not (enable_iic_outputs  and output_enables_iic_data );
 i2c_clock_t              <= not (enable_iic_outputs   and  output_enables_iic_clock );
 i2c_data_t               <= not (enable_iic_outputs  and  output_enables_iic_data );
U1: IOBUF_S_12 port map
 (I => gnd,
  T => video_encoder_sclk_t,
  O => video_encoder_sclk_i,
  IO => video_encoder_sclk_p
  );
U2: IOBUF_S_12 port map
 (I => gnd,
  T => i2c_clock_t,
  O => i2c_clock_i,
  IO => i2c_clock_p
  );
U3: IOBUF_S_12 port map
 (I => gnd,
  T => video_encoder_data_t,
  O => video_encoder_data_i,
  IO => video_encoder_data_p
  );
U4: IOBUF_S_12 port map
 (I => gnd,
  T => i2c_data_t,
  O => i2c_data_i,
  IO => i2c_data_p
  );

end  STRUCT;

