--SINGLE_FILE_TAG
-------------------------------------------------------------------------------
-- $Id: opb_gpio.vhd,v 1.2 2001/11/27 20:38:10 tise Exp $
-------------------------------------------------------------------------------
-- OPB_GPIO - entity/architecture pair
-------------------------------------------------------------------------------
--
--                  ****************************
--                  ** Copyright Xilinx, Inc. **
--                  ** All rights reserved.   **
--                  ****************************
--
-------------------------------------------------------------------------------
-- Filename:        opb_gpio.vhd
-- Version:         v1.00a
-- Description:     General Purpose I/O for OPB bus
--
-------------------------------------------------------------------------------
-- Structure:
--
--              opb_gpio.vhd
-------------------------------------------------------------------------------
-- Author:      BLT
-- History:
--  BLT             07-05-2001      -- First version
-- ^^^^^^
--      First version of GPIO.
-- ~~~~~~
--  BLT             08-08-2001      -- Added generics
-- ^^^^^^
--      Added C_GPIO_WIDTH generic to specify number of I/O bits.
--      Added C_ALL_INPUTS generic for case when all I/O are input only.
--      Added byte enable capability to read/write regs.
-- ~~~~~~
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x"
--      reset signals:                          "rst", "rst_n"
--      generics:                               "C_*"
--      user defined types:                     "*_TYPE"
--      state machine next state:               "*_ns"
--      state machine current state:            "*_cs"
--      combinatorial signals:                  "*_cmb"
--      pipelined or register delay signals:    "*_d#"
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce"
--      internal version of output port         "*_i"
--      device pins:                            "*_pin"
--      ports:                                  - Names begin with Uppercase
--      processes:                              "*_PROCESS"
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------
library IEEE;

use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
library Unisim;
use Unisim.all;
library Common_v1_00_a;
use Common_v1_00_a.all;
use Common_v1_00_a.Common_Types.all;
use Common_v1_00_a.pselect;

-------------------------------------------------------------------------------
-- Port declarations
-------------------------------------------------------------------------------

entity GPIO_MOD is
  generic (
    C_OPB_AWIDTH    : integer := 32;
    C_OPB_DWIDTH    : integer := 32;
    C_BASEADDR      : std_logic_vector := X"2000_0000";
    C_HIGHADDR      : std_logic_vector := X"2000_00FF";
    C_GPIO_WIDTH    : integer := 16;
    C_ALL_INPUTS    : integer := 0
    );
  port (
    OPB_Clk         : in  std_logic;
    OPB_Rst         : in  std_logic;

    -- OPB signals
    OPB_ABus        : in  std_logic_vector(0 to C_OPB_AWIDTH-1);
    OPB_BE          : in  std_logic_vector(0 to C_OPB_DWIDTH/8-1);
    OPB_DBus        : in  std_logic_vector(0 to C_OPB_DWIDTH-1);
    OPB_RNW         : in  std_logic;
    OPB_select      : in  std_logic;
    OPB_seqAddr     : in  std_logic;
    GPIO_DBus       : out std_logic_vector(0 to C_OPB_DWIDTH-1);
    GPIO_errAck     : out std_logic;
    GPIO_retry      : out std_logic;
    GPIO_toutSup    : out std_logic;
    GPIO_xferAck    : out std_logic;

    -- GPIO signals
    GPIO_IO_I0        :  in std_logic;
    GPIO_IO_I1        :  in std_logic;
    GPIO_IO_I2        :  in std_logic;
    GPIO_IO_I3        :  in std_logic;
    GPIO_IO_I4        :  in std_logic;
    GPIO_IO_I5        :  in std_logic;
    GPIO_IO_I6        :  in std_logic;
    GPIO_IO_I7        :  in std_logic;
    GPIO_IO_I8        :  in std_logic;
    GPIO_IO_I9        :  in std_logic;
    GPIO_IO_I10       :  in std_logic;
    GPIO_IO_I11       :  in std_logic;
    GPIO_IO_I12       :  in std_logic;
    GPIO_IO_I13       :  in std_logic;
    GPIO_IO_I14       :  in std_logic;
    GPIO_IO_I15       :  in std_logic;

    GPIO_IO_O0        : out std_logic;
    GPIO_IO_O1        : out std_logic;
    GPIO_IO_O2        : out std_logic;
    GPIO_IO_O3        : out std_logic;
    GPIO_IO_O4        : out std_logic;
    GPIO_IO_O5        : out std_logic;
    GPIO_IO_O6        : out std_logic;
    GPIO_IO_O7        : out std_logic;
    GPIO_IO_O8        : out std_logic;
    GPIO_IO_O9        : out std_logic;
    GPIO_IO_O10       : out std_logic;
    GPIO_IO_O11       : out std_logic;
    GPIO_IO_O12       : out std_logic;
    GPIO_IO_O13       : out std_logic;
    GPIO_IO_O14       : out std_logic;
    GPIO_IO_O15       : out std_logic

    --GPIO_IO_T       : out std_logic_vector(0 to C_GPIO_WIDTH-1)
    );
end entity GPIO_MOD;

-------------------------------------------------------------------------------
-- Architecture section
-------------------------------------------------------------------------------

architecture IMP of GPIO_MOD is

component GPIO_MOD_Core is
  generic (
    C_DW               : integer;
    C_AW               : integer;
    C_GPIO_WIDTH       : integer;
    C_ALL_INPUTS       : boolean
    );
  port (
    OPB_Clk            : in  std_logic;
    OPB_Rst            : in  std_logic;

    -- OPB signals
    OPB_ABus_Reg       : in  std_logic_vector(0 to C_AW-1);
    OPB_BE_Reg         : in  std_logic_vector(0 to C_DW/8-1);
    OPB_DBus_Reg       : in  std_logic_vector(0 to C_GPIO_WIDTH-1);
    OPB_RNW_Reg        : in  std_logic;
    OPB_select_Reg     : in  std_logic;
    OPB_seqAddr_Reg    : in  std_logic;
    GPIO_DBus          : out std_logic_vector(0 to C_DW-1);
    GPIO_errAck        : out std_logic;
    GPIO_retry         : out std_logic;
    GPIO_toutSup       : out std_logic;
    GPIO_xferAck       : out std_logic;
    GPIO_IO_I          : in  std_logic_vector(0 to C_GPIO_WIDTH-1);
    GPIO_IO_O          : out std_logic_vector(0 to C_GPIO_WIDTH-1);
     --GPIO_IO_T          : out std_logic_vector(0 to C_GPIO_WIDTH-1);

    -- Peripheral Select address decode
    GPIO_Select        : in  std_logic
    );
end component GPIO_MOD_Core;

component FDR is
  port (
    Q : out std_logic;
    C : in  std_logic;
    D : in  std_logic;
    R : in  std_logic
  );
end component FDR;
component pselect is
  generic (
    C_AB     : integer;
    C_AW     : integer;
    C_BAR    : std_logic_vector
    );
  port (
    A        : in   std_logic_vector(0 to C_AW-1);
    AValid   : in   std_logic;
    PS       : out  std_logic
    );
end component pselect;

signal  gnd						: std_logic;
signal  GPIO_IO_I          : std_logic_vector(0 to C_GPIO_WIDTH-1);
signal  GPIO_IO_O          : std_logic_vector(0 to C_GPIO_WIDTH-1);

-- Address decode signals
signal gpio_Select          : std_logic;
signal opb_ABus_Reg         : std_logic_vector(0 to C_OPB_AWIDTH-1);
signal opb_DBus_Reg         : std_logic_vector(0 to C_GPIO_WIDTH-1);
signal opb_BE_Reg           : std_logic_vector(0 to C_OPB_DWIDTH/8-1);
signal opb_RNW_Reg          : std_logic;
signal opb_select_Reg       : std_logic;
signal opb_seqAddr_Reg      : std_logic;

function Addr_Bits (x,y : std_logic_vector(0 to C_OPB_AWIDTH-1)) return integer is
  variable addr_nor : std_logic_vector(0 to C_OPB_AWIDTH-1);

begin

  addr_nor := x xor y;
  for i in 0 to C_OPB_AWIDTH-1 loop
    if addr_nor(i)='1' then return i;
    end if;
  end loop;
  return(C_OPB_AWIDTH);
end function Addr_Bits;

function Integer_to_Boolean (x: integer) return boolean is
begin
  if x=0 then return false;
  else return true;
  end if;
end function Integer_to_Boolean;

constant C_AB              : integer := Addr_Bits(C_HIGHADDR,C_BASEADDR);
constant C_ALL_INPUTS_BOOL : boolean := Integer_to_Boolean(C_ALL_INPUTS);

-------------------------------------------------------------------------------
-- Begin architecture
-------------------------------------------------------------------------------
begin


GPIO_IO_I(0)  <=   GPIO_IO_I0 ;  
GPIO_IO_I(1)  <=   GPIO_IO_I1 ;  
GPIO_IO_I(2)  <=   GPIO_IO_I2 ;  
GPIO_IO_I(3)  <=   GPIO_IO_I3 ;  
GPIO_IO_I(4)  <=   GPIO_IO_I4 ;  
GPIO_IO_I(5)  <=   GPIO_IO_I5 ;  
GPIO_IO_I(6)  <=   GPIO_IO_I6 ;  
GPIO_IO_I(7)  <=   GPIO_IO_I7 ;  
GPIO_IO_I(8)  <=   GPIO_IO_I8 ;  
GPIO_IO_I(9)  <=   GPIO_IO_I9 ;  
GPIO_IO_I(10) <=  GPIO_IO_I10 ; 
GPIO_IO_I(11) <=  GPIO_IO_I11 ; 
GPIO_IO_I(12) <=  GPIO_IO_I12 ; 
GPIO_IO_I(13) <=  GPIO_IO_I13 ; 
GPIO_IO_I(14) <=  GPIO_IO_I14 ;
GPIO_IO_I(15) <=  GPIO_IO_I15 ;
     

GPIO_IO_O0 <= GPIO_IO_O(0);
GPIO_IO_O1 <= GPIO_IO_O(1);
GPIO_IO_O2 <= GPIO_IO_O(2);
GPIO_IO_O3 <= GPIO_IO_O(3);
GPIO_IO_O4 <= GPIO_IO_O(4);
GPIO_IO_O5 <= GPIO_IO_O(5);
GPIO_IO_O6 <= GPIO_IO_O(6);
GPIO_IO_O7 <= GPIO_IO_O(7);
GPIO_IO_O8 <= GPIO_IO_O(8);
GPIO_IO_O9 <= GPIO_IO_O(9);
GPIO_IO_O10 <= GPIO_IO_O(10);
GPIO_IO_O11 <= GPIO_IO_O(11);
GPIO_IO_O12 <= GPIO_IO_O(12);
GPIO_IO_O13 <= GPIO_IO_O(13);
GPIO_IO_O14 <= GPIO_IO_O(14);
GPIO_IO_O15 <= GPIO_IO_O(15);


  -----------------------------------------------------------------------------
  -- Register all the OPB input signals
  -----------------------------------------------------------------------------
  WRDBUS_FF_GENERATE: for i in 0 to C_GPIO_WIDTH-1 generate
    WRDBUS_FF_I: FDR
    port map (
      Q  => opb_DBus_Reg(i),                       -- [out]
      C  => OPB_Clk,                               -- [in]
      D  => OPB_DBus(i+C_OPB_DWIDTH-C_GPIO_WIDTH), -- [in]
      R  => OPB_Rst                                -- [in]
    );
  end generate WRDBUS_FF_GENERATE;

  ABUS_FF_GENERATE: for i in 0 to C_OPB_AWIDTH-1 generate
    ABUS_FF_I: FDR
    port map (
      Q  => opb_ABus_Reg(i),   -- [out]
      C  => OPB_Clk,           -- [in]
      D  => OPB_ABus(i),       -- [in]
      R  => OPB_Rst            -- [in]
    );
  end generate ABUS_FF_GENERATE;

  BE_FF_GENERATE: for i in 0 to C_OPB_DWIDTH/8-1 generate
    BE_FF_I: FDR
    port map (
      Q  => opb_BE_Reg(i),  -- [out]
      C  => OPB_Clk,        -- [in]
      D  => OPB_BE(i),      -- [in]
      R  => OPB_Rst         -- [in]
    );
  end generate BE_FF_GENERATE;

  RNW_FF_I: FDR
  port map (
    Q  => opb_RNW_Reg,      -- [out]
    C  => OPB_Clk,          -- [in]
    D  => OPB_RNW,          -- [in]
    R  => OPB_Rst           -- [in]
  );
  SELECT_FF_I: FDR
  port map (
    Q  => opb_select_Reg,   -- [out]
    C  => OPB_Clk,          -- [in]
    D  => OPB_select,       -- [in]
    R  => OPB_Rst           -- [in]
  );
  SEQADDR_FF_I: FDR
  port map (
    Q  => opb_seqAddr_Reg,  -- [out]
    C  => OPB_Clk,          -- [in]
    D  => OPB_seqAddr,      -- [in]
    R  => OPB_Rst           -- [in]
  );

  -----------------------------------------------------------------------------
  -- Address decode
  -----------------------------------------------------------------------------

  GPIO_Select_I: pselect
    generic map (
      C_AB     => C_AB,
      C_AW     => C_OPB_AWIDTH,
      C_BAR    => C_BASEADDR )
    port map (
      A        => opb_ABus_Reg,    -- [in]
      AValid   => opb_Select_Reg,  -- [in]
      PS       => gpio_Select);    -- [out]

  GPIO_CORE_I: GPIO_MOD_Core
    generic map (
      C_DW               => C_OPB_DWIDTH,
      C_AW               => C_OPB_AWIDTH,
      C_GPIO_WIDTH       => C_GPIO_WIDTH,
      C_ALL_INPUTS       => C_ALL_INPUTS_BOOL
      )
    port map (
      -- OPB signals
      OPB_Clk            => OPB_Clk,          --[in]
      OPB_Rst            => OPB_Rst,          --[in]
      OPB_ABus_Reg       => opb_ABus_Reg,     --[in]
      OPB_BE_Reg         => opb_BE_Reg,       --[in]
      OPB_DBus_Reg       => opb_DBus_Reg,     --[in]
      OPB_RNW_Reg        => opb_RNW_Reg,      --[in]
      OPB_select_Reg     => opb_select_Reg,   --[in]
      OPB_seqAddr_Reg    => opb_seqAddr_Reg,  --[in]
      GPIO_DBus          => GPIO_DBus,        --[out]
      GPIO_errAck        => GPIO_errAck,      --[out]
      GPIO_retry         => GPIO_retry,       --[out]
      GPIO_toutSup       => GPIO_toutSup,     --[out]
      GPIO_xferAck       => GPIO_xferAck,     --[out]

      -- GPIO signals
      GPIO_IO_I          => GPIO_IO_I,        --[in]
      GPIO_IO_O          => GPIO_IO_O,        --[out]
      --GPIO_IO_T          => GPIO_IO_T,        --[out]

      -- Peripheral Select address decode
      GPIO_Select        => gpio_Select       --[in]
      );

end architecture IMP;

